package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jeecg.modules.ctop.entity.ByteDanceAdvertisePlan;
import org.jeecg.modules.ctop.entity.BytedanceDeliveryTargetTemplate;
import org.jeecg.modules.ctop.entity.BytedanceUrlInfo;
import org.jeecg.modules.ctop.entity.CtopOauthToken;
import org.jeecg.modules.ctop.mapper.ByteDanceAdvertisePlanMapper;
import org.jeecg.modules.ctop.mapper.BytedanceDeliveryTargetTemplateMapper;
import org.jeecg.modules.ctop.mapper.BytedanceUrlInfoMapper;
import org.jeecg.modules.ctop.service.IByteDanceAdvertisePlanService;
import org.jeecg.modules.ctop.service.ICtopOauthTokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 今日头条广告计划信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Service
public class ByteDanceAdvertisePlanServiceImpl extends ServiceImpl<ByteDanceAdvertisePlanMapper, ByteDanceAdvertisePlan> implements IByteDanceAdvertisePlanService {
    @Autowired
    private BytedanceUrlInfoMapper urlInfoMapper;

    @Autowired
    private BytedanceDeliveryTargetTemplateMapper deliveryTargetTemplateMapper;

    @Autowired
    private ICtopOauthTokenService tokenService;

    @Autowired
    private ByteDanceAdvertisePlanMapper planMapper;

    @Override
    public Map<String, Object> advertiserPlanCreate(CtopOauthToken token, Long campaignId, String dataString) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONObject data = JSONObject.parseObject(dataString);
        data.put("advertiser_id", token.getAccountId());
        data.put("campaign_id", campaignId);
        JSONObject result = createAd(data, token.getAccessToken());
        Integer code = result.getInteger("code");
        String message = result.getString("message");
        if (null == code || code != 0) {
            log.info("广告计划创建失败,广告主id:{},message:{}", token.getAdvertiserId(), message);
            resultMap.put("code", -1);
            resultMap.put("message", message);
            return resultMap;
        }
        JSONObject dataObject = result.getJSONObject("data");
        Long adId = dataObject.getLong("ad_id");
        ByteDanceAdvertisePlan plan = new ByteDanceAdvertisePlan(token.getAdvertiserId(), adId, data);

        planMapper.insert(plan);
        resultMap.put("planId", adId);
        resultMap.put("code", 0);
        resultMap.put("message", "广告计划创建成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdConvertList(Long accountId, Long targetId) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId + "");
        BytedanceDeliveryTargetTemplate targetTemplate = deliveryTargetTemplateMapper.selectById(targetId);
        return getAdConvert(token, targetTemplate);
    }

    @Override
    public Map<String, Object> getAdConvertListByUrlId(Long accountId, Long urlId) {
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(accountId + "");
        BytedanceUrlInfo urlInfo = urlInfoMapper.selectById(urlId);
        return getAdConvertByUrlInfo(token, urlInfo);
    }

    private Map<String, Object> getAdConvertByUrlInfo(CtopOauthToken token, BytedanceUrlInfo urlInfo) {
        String accessToken = token.getAccessToken();
        final Long advertiserId = token.getAccountId();
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/ad_convert/select/";

        // 请求参数
        Map data = new HashMap() {
            {
                put("advertiser_id", advertiserId);
            }
        };
        //落地页或IOS下载链接
        if ("page".equals(urlInfo.getType()) || "page".equals(urlInfo.getType())) {
            data.put("external_url", urlInfo.getUrl());
        }
        //安卓下载链接
        if ("android".equals(urlInfo.getType())) {
            data.put("package_name", urlInfo.getPackageName());
        }

        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };
        httpEntity.setHeader("Access-Token", accessToken);
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(url));
            httpEntity.setEntity(new StringEntity(JSONObject.toJSONString(data), ContentType.APPLICATION_JSON));
            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject getAdConvert(CtopOauthToken token, BytedanceDeliveryTargetTemplate targetTemplate) {
        String accessToken = token.getAccessToken();
        final Long advertiserId = token.getAccountId();
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/ad_convert/select/";

        // 请求参数
        Map data = new HashMap() {
            {
                put("advertiser_id", advertiserId);
            }
        };
        //落地页
        if ("EXTERNAL_URL".equals(targetTemplate.getDownloadType())) {
            data.put("external_url", targetTemplate.getDownloadUrl());
        }
        //下载链接
        if ("DOWNLOAD_URL".equals(targetTemplate.getDownloadType())) {
            if ("ios".equals(targetTemplate.getDownloadMode())) {
                data.put("download_url", targetTemplate.getDownloadUrl());
            }
            if ("android".equals(targetTemplate.getDownloadMode())) {
                data.put("package_name", targetTemplate.getPackageName());
            }
        }


        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };

        httpEntity.setHeader("Access-Token", accessToken);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(url));
            httpEntity.setEntity(new StringEntity(JSONObject.toJSONString(data), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }


    public JSONObject createAd(JSONObject data, String token) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_ad_create");
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);
        httpEntity.setHeader("Access-Token", token);
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(data.toJSONString(), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }
}
