package org.jeecg.modules.kuaishou.service.impl;

import cn.com.ctop.common.utils.Check;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import constant.KuaishouInterfaceConstant;
import org.jeecg.modules.ctop.entity.CtopOauthToken;
import org.jeecg.modules.ctop.service.ICtopOauthTokenService;
import org.jeecg.modules.kuaishou.entity.KuaiShouCreateAppTemplate;
import org.jeecg.modules.kuaishou.mapper.KuaiShouCreateAppTemplateMapper;
import org.jeecg.modules.kuaishou.service.IKuaiShouCreateAppTemplateService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 创建应用
 * @Author: jeecg-boot
 * @Date: 2019-08-06
 * @Version: V1.0
 */
@Service
public class KuaiShouCreateAppTemplateServiceImpl extends ServiceImpl<KuaiShouCreateAppTemplateMapper, KuaiShouCreateAppTemplate> implements IKuaiShouCreateAppTemplateService {
    @Autowired
    private KuaiShouCreateAppTemplateMapper appTemplateMapper;
    @Autowired
    private ICtopOauthTokenService oauthTokenService;
    @Autowired
    private IKuaishouInterfaceService interfaceService;

    @Override
    public Map<String, Object> insert(KuaiShouCreateAppTemplate appTemplate) {
        Map<String, Object> resultMap = new HashMap<>();
        try {
            Long accountId = appTemplate.getAccountId();
            CtopOauthToken cTopOauthToken = oauthTokenService.getOauthTokenByAccountId(String.valueOf(accountId));
            if (Check.isNull(cTopOauthToken)) {
                throw new Exception("未获取到token信息");
            }
            String accessToken = cTopOauthToken.getAccessToken();
            String loginId = appTemplate.getLoginId();
            String appVersion = appTemplate.getAppVersion();
            QueryWrapper<KuaiShouCreateAppTemplate> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("app_version", appVersion);
            queryWrapper.eq("login_id", loginId);
            queryWrapper.orderByDesc("create_time");
            queryWrapper.last("limit 1");
            KuaiShouCreateAppTemplate kuaiShouCreateAppTemplate = appTemplateMapper.selectOne(queryWrapper);
            if (!Check.isNull(kuaiShouCreateAppTemplate)) {
                resultMap.put("success", false);
                resultMap.put("message", "应用标记重复");
                resultMap.put("code", -1);
            } else {
                if (!Check.isNull(appTemplate.getFile())) {
                    appTemplate.setFile(KuaishouInterfaceConstant.HTTPS_PREFIX + appTemplate.getFile());
                }
                if (!Check.isNull(appTemplate.getImageUrl())) {
                    appTemplate.setImageUrl(KuaishouInterfaceConstant.HTTPS_PREFIX + appTemplate.getImageUrl());
                }
                appTemplateMapper.insert(appTemplate);

                Thread thread = new Thread() {
                    @Override
                    public void run() {
                        JSONObject apkJson = new JSONObject();
                        String platform = appTemplate.getPlatform();
                        apkJson.put("app_name", appTemplate.getAppName());
                        apkJson.put("app_version", appTemplate.getAppVersion());
                        apkJson.put("platform", platform);
                        if (!Check.isNull(appTemplate.getPackageName())) {
                            apkJson.put("package_name", appTemplate.getPackageName());
                        }
                        if (!Check.isNull(appTemplate.getImageUrl()) && ("1".equals(platform) || "2".equals(platform))) {
                            Map<String, Object> imageMap = interfaceService.imageUpload(accountId, accessToken, (String) appTemplate.getImageUrl(), 1);
                            apkJson.put("image_token", imageMap.get("imageToken"));
                        }
                        if ("1".equals(platform) && appTemplate.getUploadType() == 1) {
                            apkJson.put("file", appTemplate.getFile());
                            interfaceService.appCreate(accountId, accessToken, apkJson);
                        } else {
                            apkJson.put("url", appTemplate.getUrl());
                            interfaceService.urlAppCreate(accountId, accessToken, apkJson);
                        }
                    }

                };
                thread.start();
                resultMap.put("success", true);
                resultMap.put("message", "创建应用成功");
                resultMap.put("code", 0);
            }

        } catch (Exception e) {
            resultMap.put("success", false);
            resultMap.put("message", "创建应用失败");
            resultMap.put("code", -1);
        }
        return resultMap;

    }
}
