package org.jeecg.modules.kuaishou.service.impl;

import cn.com.ctop.common.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import constant.KuaishouInterfaceConstant;
import org.jeecg.common.util.encryption.AesEncryptUtil;
import org.jeecg.modules.ctop.entity.CtopOauthToken;
import org.jeecg.modules.ctop.service.ICtopOauthTokenService;
import org.jeecg.modules.kuaishou.entity.KuaiShouImage;
import org.jeecg.modules.kuaishou.entity.KuaiShouVideo;
import org.jeecg.modules.kuaishou.mapper.KuaiShouImageMapper;
import org.jeecg.modules.kuaishou.service.IKuaiShouImageService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.List;

/**
 * @Description: 快手-本地图片保存
 * @Author: jeecg-boot
 * @Date: 2019-08-14
 * @Version: V1.0
 */
@Service
public class KuaiShouImageServiceImpl extends ServiceImpl<KuaiShouImageMapper, KuaiShouImage> implements IKuaiShouImageService {

    @Autowired
    private KuaiShouImageMapper imageMapper;

    @Autowired
    private IKuaishouInterfaceService interfaceService;
    @Autowired
    private ICtopOauthTokenService oauthTokenService;

    @Override
    public void insert(JSONObject requestJson) throws Exception {
        JSONArray imageArr = requestJson.getJSONArray("localImageUrl");
        Long accountId = requestJson.getLong("accountId");
        CtopOauthToken cTopOauthToken = oauthTokenService.getOauthTokenByAccountId(String.valueOf(accountId));
        if (Check.isNull(cTopOauthToken)) {
            throw new Exception("未获取到token信息");
        }
        String accessToken = cTopOauthToken.getAccessToken();
        if (!Check.isNull(imageArr)) {
            for (int i = 0; i < imageArr.size(); i++) {
                KuaiShouImage image = new KuaiShouImage();
                String imageUrl = imageArr.get(i).toString();
                String fileName = imageUrl.substring(imageUrl.lastIndexOf("/") + 1);
                String fileNameStr = null;
                if (!Check.isNull(fileName)) {
                    String[] split = fileName.split("-");
                    fileNameStr = split[0];
                }
                image.setImageName(AesEncryptUtil.getUrlDecoderString(fileNameStr));
                image.setLoginId(requestJson.getString("loginId"));
                image.setMaterialType(requestJson.getString("materialType"));
                image.setPositionType(requestJson.getString("positionType"));
                image.setAccountId(accountId);
                String imageUrlStr;
                if (!imageUrl.contains(KuaishouInterfaceConstant.HTTPS_PREFIX + imageUrl)) {
                    imageUrlStr = KuaishouInterfaceConstant.HTTPS_PREFIX + imageUrl;
                } else {
                    imageUrlStr = imageUrl;
                }
                image.setLocalUrl(imageUrlStr);
                imageMapper.insert(image);

                Thread thread = new Thread() {
                    @Override
                    public void run() {
                        interfaceService.imageUpload(accountId, accessToken, imageUrlStr, 0);
                    }
                };
                thread.start();
            }
        }

    }

    @Override
    public String localInsert(JSONObject imageJson) {
        String url = imageJson.getString("url");
        String positionType = imageJson.getString("positionType");
        String videoUploadType = imageJson.getString("videoUploadType");
        String loginId = imageJson.getString("loginId");
        Long accountId = imageJson.getLong("accountId");

        QueryWrapper<KuaiShouImage> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("local_url", url);
        queryWrapper.eq("account_id", accountId);
        queryWrapper.eq("position_type", positionType);
        queryWrapper.eq("material_type", videoUploadType);
        queryWrapper.eq("login_id", loginId);
        queryWrapper.orderByDesc("create_time");
        queryWrapper.last("limit 1");
        KuaiShouImage imageLocal = imageMapper.selectOne(queryWrapper);
        if (Check.isNull(imageLocal)) {
            KuaiShouImage image = new KuaiShouImage();
            String fileName = url.substring(url.lastIndexOf("/") + 1);
            String fileNameStr = null;
            if (!Check.isNull(fileName)) {
                String[] split = fileName.split("-");
                fileNameStr = split[0];
            }
            image.setLoginId(loginId);
            image.setImageName(AesEncryptUtil.getUrlDecoderString(fileNameStr));
            image.setMaterialType(videoUploadType);
            image.setPositionType(positionType);
            if (!url.contains("https:")) {
                String imageUrl = KuaishouInterfaceConstant.HTTPS_PREFIX + url;
                image.setLocalUrl(imageUrl);
            } else {
                image.setLocalUrl(url);
            }
            image.setAccountId(accountId);

            imageMapper.insert(image);
            return image.getLocalUrl().toString();
        }

        return url;
    }

    @Override
    public JSONArray selectImageList(QueryWrapper<KuaiShouImage> queryWrapper) {
        List<KuaiShouImage> kuaiShouImages = imageMapper.selectList(queryWrapper);
        JSONArray jsonArray = new JSONArray();
        if (!Check.isNull(kuaiShouImages)) {
            for (KuaiShouImage image : kuaiShouImages) {
                if (!Check.isNull(image)) {
                    JSONObject json = new JSONObject();
                    json.put("id", image.getId());
                    json.put("src", image.getLocalUrl());
                    json.put("msrc", image.getLocalUrl());
                    json.put("w", 600);
                    json.put("h", 400);
                    json.put("name", image.getImageName());
                    jsonArray.add(json);
                }
            }
        }

        return jsonArray;
    }
}
